/* 
 * Core Interface for OpenAntiVirus ScannerDaemon			
 *
 * Copyright (C) Rainer Link, 2001-2002
 *               OpenAntiVirus.org <rainer@openantivirus.org>
 *
 * Credits to W. Richard Stevens - RIP
 * 
 * This software is licensed under the GNU General Public License (GPL)
 * See: http://www.gnu.org/copyleft/gpl.html
 *
*/

#include <includes.h>
#include "vscan-oav_core.h"
 

/* print a message via syslog */
void vscan_syslog(char *printMessage, ...)
{
        char printMsg[512];

        va_list argptr;
        va_start(argptr, printMessage);
        vsnprintf(printMsg, sizeof(printMsg)-1, printMessage, argptr);
        va_end(argptr);

	syslog(SYSLOG_PRIORITY, "%s", printMsg);
}



/* initialise socket to ScannerDaemon
   returns -1 on error or the socket descriptor */
int vscan_oav_init(void)
{

	int sockfd;
	struct sockaddr_in servaddr;

	/* create socket */
        if (( sockfd = socket(AF_INET, SOCK_STREAM, 0)) < 0 ) {
               vscan_syslog("ERROR: can not create socket!\n");
               return -1; 
        }

	bzero(&servaddr, sizeof(servaddr));
        servaddr.sin_family = AF_INET;
        servaddr.sin_port = htons(OAV_PORT);

	/* hm, inet_pton may not exist on all systems - FIXME ! */
        if ( inet_pton(AF_INET, OAV_IP, &servaddr.sin_addr) <= 0 ) {
                vscan_syslog("ERROR: inet_pton failed!\n");
                return -1;
	}

	/* connect to socket */
	if ( connect(sockfd, (struct sockaddr *) &servaddr, sizeof(servaddr)) < 0 )
        {
                vscan_syslog("ERROR: can not connect to ScannerDaemon!\n");
                return -1;
        }


	return sockfd;

}

/*
void vscan_oav_syslog(char *syslogMessage, ...)
{
        char logMsg[512];

        va_list argptr;
        va_start(argptr, syslogMessage);
        vsnprintf(logMsg, sizeof(logMsg)-1, syslogMessage, argptr);
        va_end(argptr);

        syslog(SYSLOG_PRIORITY, "%s\n", logMsg);
}

*/

/*
  If virus is found, logs the filename/virusname into syslog
*/
void vscan_oav_log_virus(char *infected_file, char *result)
{
	char *str;

	/* remove "FOUND: " from the result string to get only the virus name */
	str = result;
	str+= 7;

        vscan_syslog("Scan result: '%s' infected with virus '%s'", infected_file, str);
        
}



/*
  Scans a file (*FILE*, not a directory - keep that in mind) for a virus
  Expects socket descriptor and file name to scan for
  Return -1 on error, 0 if no virus was found, 1 if a virus was found 
*/
int vscan_oav_scanfile(int sockfd, char *scan_file)
{
	size_t len;
	char recvline[MAXLINE + 1];
	pstring oavCommand;

	FILE *fpin, *fpout;

	fpin = fdopen(sockfd, "r");
	if ( fpin == NULL ) {
		vscan_syslog("ERROR: can not open stream for reading - %s", strerror(errno));
		return -1;
	}

	fpout = fdopen(sockfd, "w");
	if ( fpout == NULL ) {
		vscan_syslog("ERROR: can not open stream for writing - %s", strerror(errno));
		return -1;
	}

        #if VSCAN_OAV_VERBOSE_FILE_LOGGING==1
        vscan_syslog("Scanning file : '%s'", scan_file);
        #endif

	/* ScannerDaemon expects "SCAN <filename>\n" */
	/* what about if the <filename> itself contains '\n'? */
        fstrcpy(oavCommand, "SCAN ");
        fstrcat(oavCommand, scan_file);
        fstrcat(oavCommand, "\n");

	/* write to socket */
	/* NOTE: what happens if scan_file is very long? */
	len = strlen(oavCommand);
	if ( len < 0 ) {
		/* overflow? */
		vscan_syslog("ERROR: file name is too long!");
		return -1;
	}	

	/* write to socket-stream */
	if ( fputs(oavCommand, fpout) == EOF) {
		vscan_syslog("ERROR: can not send file name to ScannerDaemon!");
		return -1;
	}

	/* hum, instead of flush()ing, use setvbuf to set line-based buffering? */
	if ( fflush(fpout) == EOF ) {
		vscan_syslog("ERROR: can not flush output stream -%s", strerror(errno));
	}

	/* read from socket-stream */
	if ( fgets(recvline, MAXLINE, fpin) == NULL ) {
		vscan_syslog("ERROR: can not get result from ScannerDaemon!");
		return -1;
	}

	/* Response from ScannerDaemon is either
	   - OK                  file scanned and not infected
	   - FOUND: <virusname>  file scanned and infected
	   - ERROR		 sth went wrong, i.e. file not found
	*/
	if ( strncmp("FOUND", recvline, 5) == 0   ) {
		/* virus found */
		vscan_oav_log_virus(scan_file, recvline);
		return 1;
	} else if ( strncmp("ERROR", recvline, 5) == 0  ) {
		/* ERROR */
		#if VSCAN_OAV_VERBOSE_FILE_LOGGING==1
		vscan_syslog("file %s not found or not readable", scan_file);
		#endif
	} else if ( strncmp("OK", recvline, 2) == 0 ) {
		/* OK */
		#if VSCAN_OAV_VERBOSE_FILE_LOGGING==1
		vscan_syslog("file %s is clean", scan_file);
		#endif
        } else {
		/* unknown response */
		vscan_syslog("unknown response from ScannerDaemon while scanning %s!", scan_file);
	}	

        if ( fclose(fpin) == EOF ) {
                vscan_syslog("ERROR: can not close input stream - %s", strerror(errno));
        }

        if ( fclose(fpout) == EOF ) {
                vscan_syslog("ERROR: can not close output stream - %s", strerror(errno));
        }


	return 0;
	
}

/*
  close socket
*/
void vscan_oav_end(int sockfd)
{
	/* sockfd == -1 indicates an error while connecting to socket */
	if ( sockfd >= 0 ) {
		close(sockfd);
	}

}
